/*
 * Copyright 2008, by Danil S. Trommel, Avance, The Netherlands
 * 
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.IO;
using System.Drawing;
using System.Xml;
using System.Windows.Forms;

using miniTeach.Teach;
using miniTeach.Tools;


namespace miniTeach.Settings
{
	/// <summary>
	/// Description of TeachSettings (Singleton pattern)
	/// </summary>
	public sealed class TeachSettings
	{
		
		
		#region Singleton Pattern
		
		private static TeachSettings instance = new TeachSettings();
		
		public static TeachSettings Instance {
			get {
				return instance;
			}
		}
		
		private TeachSettings()
		{
			//TODO: default settings, in case of non-existing settings file
		}
		
		#endregion

		
		
		#region General Fields
		
		public const int TextOffset = 3;
		public const string FontName = "Arial";
		public const string VersionInfo = "miniTeach v1.0 Beta";
		
		private bool m_IsWinCE;
		public bool IsWinCE {
			get { return m_IsWinCE; }
			set { m_IsWinCE = value; }
		}
		
		#endregion
		
		
		
		#region Metrics
		
		private int m_EmTextHeight;
		public int EmTextHeight {
			get { return m_EmTextHeight; }
			set { m_EmTextHeight = value; }
		}
		
		private int m_EmTextHeightSmall;
		public int EmTextHeightSmall {
			get { return m_EmTextHeightSmall; }
			set { m_EmTextHeightSmall = value; }
		}

		private FontMetricFactors m_FontMetricFactors;
		public FontMetricFactors FontMetricFactors {
			get { return m_FontMetricFactors; }
			set { m_FontMetricFactors = value; }
		}
		
		private FontPixelDimensions m_FontPixelDimensions;
		public FontPixelDimensions FontPixelDimensions {
			get { return m_FontPixelDimensions; }
			set { m_FontPixelDimensions = value; }
		}
		
		#endregion
				
		
		#region Paths
		
		private string m_ApplicationPath;	//Path to program executable
		public string ApplicationPath {
			get { return m_ApplicationPath; }
			set { m_ApplicationPath = value; }
		}
		
		private string m_SettingsPath;	//Path to the settings file
		public string SettingsPath {
			get { return m_SettingsPath; }
			set { m_SettingsPath = value; }
		}
		
		#endregion
				
		
		#region Stored Variables
		
		private int m_FontSize;
		public int FontSize {
			get { return m_FontSize; }
			set { m_FontSize = value; }
		}
		
		private int m_FlashTime;
		public int FlashTime {
			get { return m_FlashTime; }
			set { m_FlashTime = value; }
		}
		
		//Direct access to the structure
		public ExamSettings ExamSettings;
		
		#endregion

		
		
		#region Settings Loading / Storage

		public void LoadSettings(string nPath)
		{
			SettingsPath = ApplicationPath + nPath;
			if (File.Exists(SettingsPath))
			{
				try {
					//Generate new object and load the document
					System.Xml.XmlDocument xdoc = new XmlDocument();
					xdoc.Load(SettingsPath);
					XmlElement root = xdoc.DocumentElement;
					System.Xml.XmlNodeList nodeList = root.ChildNodes.Item(0).ChildNodes;
					//Load the things
					ExamSettings.Direction = (ExamDirection)Convert.ToInt32(nodeList.Item(0).Attributes["value"].Value);
					ExamSettings.Check = (ExamCheck)Convert.ToInt32(nodeList.Item(1).Attributes["value"].Value);
					ExamSettings.Method = (ExamMethod)Convert.ToInt32(nodeList.Item(2).Attributes["value"].Value);
					m_FontSize = Convert.ToInt32(nodeList.Item(3).Attributes["value"].Value);
					m_FlashTime = Convert.ToInt32(nodeList.Item(4).Attributes["value"].Value);
				}
				catch (Exception ex) {
					//No real problem; settings will default to standard settings.
					MessageBox.Show("LoadSettings() Error: Settings file doesn't seem valid or access is denied");
				}
			}
		}

		public void SaveSettings()
		{
			//TODO: replace the XML writer (and reader) with a more lightweight coded solution
			//(e.g. System.ConfigurationSettings.AppSettings method)
			
			XmlTextWriter tw = new XmlTextWriter(SettingsPath, System.Text.UTF8Encoding.UTF8);
			tw.WriteStartDocument();
			tw.WriteStartElement("configuration");
			tw.WriteStartElement("appSettings");

			writeXMLElement(tw, "Direction", Convert.ToByte(ExamSettings.Direction).ToString());
			writeXMLElement(tw, "Check", Convert.ToByte(ExamSettings.Check).ToString());
			writeXMLElement(tw, "Method", Convert.ToByte(ExamSettings.Method).ToString());
			writeXMLElement(tw, "FontSize", FontSize.ToString());
			writeXMLElement(tw, "FlashTime", FlashTime.ToString());

			tw.WriteEndElement();
			tw.WriteEndElement();
			tw.Close();
		}
		
		private void writeXMLElement(XmlTextWriter tw, string elementName, string elementValue)
		{
			tw.WriteStartElement("add");
			tw.WriteStartAttribute("key", string.Empty);
			tw.WriteRaw(elementName);
			tw.WriteEndAttribute();
			tw.WriteStartAttribute("value", string.Empty);
			tw.WriteRaw(elementValue);
			tw.WriteEndAttribute();
			tw.WriteEndElement();
		}
		
		#endregion

		
		
	}
}
