/*
 * Copyright 2008, by Danil S. Trommel, Avance, The Netherlands
 * 
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Xml;
using System.Collections.Generic;

namespace miniTeach.Teach
{
	public class TeachExam
	{
		private bool m_LastAnswerCorrect;
		public bool LastAnswerCorrect {
			get { return m_LastAnswerCorrect; }
		}
		
		private ExamFase m_Fase;
		public ExamFase Fase {
			get { return m_Fase; }
			set { m_Fase = value; }
		}


		private ExamSettings m_Settings;
		public ExamSettings Settings {
			get { return m_Settings; }
			set { m_Settings = value; }
		}
		
		private Question[] m_ExamQuestions;			//Questions of the exam
		private Queue<int> m_QuestionNumberQueue;	//Stores the order in which the questions are displayed
		private int m_NumberOfCorrectAnswers;		//Counter for the correct answered questions
		private int m_NumberOfWrongAnswers;			//Guess what!? This is the counter for the questions wrongly answered! Heck Yeah!
		private Random m_Randomizer;

		
		public TeachExam(Question[] questions)
		{
			//Check if there any questions...
			if (questions.Length > 0)
			{
				m_ExamQuestions = questions;
			}
			else
			{
				//...there should be!
				throw new Exception("Exam() Error; no questions are added!");
			}
		}

		public void Restart(ExamSettings examSettings)
		{
			m_Randomizer = new Random(DateTime.Now.Millisecond);	//Do NOT put this inside the constructor ;-)
			Settings = examSettings;

			FillQueue();
			
			//Reset the counters
			m_NumberOfCorrectAnswers = 0;
			m_NumberOfWrongAnswers = 0;
		}
		
		public float CalculateGrade()
		{
			float grade = (float)m_NumberOfCorrectAnswers / (m_NumberOfCorrectAnswers + m_NumberOfWrongAnswers);
			return (float)Math.Round(Math.Max(1, 10 * grade), 1);
		}
		
		public string GetProgressString()
		{
			int totaldone = m_NumberOfCorrectAnswers + m_NumberOfWrongAnswers + 1;	// +1 because of the one question currently on screen
			int inqueue = m_QuestionNumberQueue.Count;
			
			return String.Format("{0:0} / {1:0}", totaldone, (totaldone+inqueue));
		}

		public Question GetNextQuestion()
		{
			if (m_QuestionNumberQueue.Count > 0)
			{
				m_Fase = ExamFase.ShowingQuestion;	//update status
				//Get question
				int nextQuestionNumber = m_QuestionNumberQueue.Peek();	//only peek, do not yet dequeue
				Question question = m_ExamQuestions[nextQuestionNumber];
				//Set Question direction
				question.QuestionDirection = GetQuestionDirection();
				return question;
			}
			else
			{
				//Error in parent code; should check ExamFinished function! 
				throw new Exception("GetNextQuestion() Error; no questions left");
			}
		}
		
		public bool ExamFinished()
		{
			if (m_QuestionNumberQueue.Count == 0)
			{
				this.Fase = ExamFase.AskingRestart;
				return true;
			}			
			return false;
		}

		public void QuestionCorrectlyAnswered(bool correctAnswered)
		{
			m_LastAnswerCorrect = correctAnswered;
			
			if (correctAnswered)
			{
				m_NumberOfCorrectAnswers += 1;
				m_QuestionNumberQueue.Dequeue();
			}
			else
			{
				m_NumberOfWrongAnswers += 1;
				m_QuestionNumberQueue.Enqueue(m_QuestionNumberQueue.Dequeue());		//enqueue again!
			}
		}
		
		private QuestionDirection GetQuestionDirection()
		{
			if (Settings.Direction == ExamDirection.Foreign)
			{
				return QuestionDirection.Foreign;
			}
			else if (Settings.Direction == ExamDirection.Native)
			{
				return QuestionDirection.Native;
			}
			else
			{
				//Determine a random direction
				if (m_Randomizer.Next(1, 11) > 5)
				{
					return QuestionDirection.Foreign;
				}
				else
				{
					return QuestionDirection.Native;
				}
			}
		}
		
		private void FillQueue()
		{
			int[] shuffledQuestionNumbers = ShuffleQuestionNumbers();
			m_QuestionNumberQueue = new Queue<int>(shuffledQuestionNumbers);
		}
		
		private int[] ShuffleQuestionNumbers()
		{
			int[] arrayQuestionNumbers = new int[m_ExamQuestions.Length];
			
			for (int index = 0; index < arrayQuestionNumbers.Length; index++)
			{
				arrayQuestionNumbers[index] = index;
			}
			
			for (int index = arrayQuestionNumbers.Length-1; index > 0; index--)
			{
				int questionnumber = m_Randomizer.Next(index+1);
				int tempnumber = arrayQuestionNumbers[index];
				arrayQuestionNumbers[index] = arrayQuestionNumbers[questionnumber];
				arrayQuestionNumbers[questionnumber] = tempnumber;
			}
			
			return arrayQuestionNumbers;
		}
		
	}
}
