/*
 * Copyright 2008, by Danil S. Trommel, Avance, The Netherlands
 * 
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Drawing.Imaging;

using miniTeach.Settings;

namespace miniTeach.Tools
{
	
	
	public struct FontMetricFactors
	{
		//Conversion Factors
		public float Cell;
		public float CharHeight;
		public float CharWidth;
		public float Line;
		public float Descent;
		public float Ascent;
		public float InternalLeading;
		public float ExternalLeading;
	}
	
	public struct FontPixelDimensions
	{
		public float LineHeight;
		public float CellHeight;
		public float CharHeight;
		public float TextHeight;
		public float TextSmallHeight;
		public float TextSmallOffset;
		public float AverageCharWidth;
		public float InternalLeading;
		public float ExternalLeading;
		public float Ascent;
		public float Descent;
	}
	
	/// <summary>
	/// Description of FontMetricTool.
	/// </summary>
	public static class FontMetricTool
	{
		
		
		#region P/Invoke functions for FontMetrics
		
		//  Below is a description of all keywords (functions and variables) used...
		
		//	Ascent  			 the ascent of characters in the font.
		// 	AverageCharWidth 	 the average width of characters in the font.
		// 	BreakChar 			 the character used to define word breaks for text justification.
		// 	CharSet 			 the character set of the font.
		// 	DefaultChar 		 the character to be substituted for characters not in the font.
		// 	Descent 			 the descent of characters in the font.
		// 	DigitizedAspectX 	 the horizontal aspect of the device for which the font was designed.
		// 	DigitizedAspectY 	 the vertical aspect of the device for which the font was designed.
		// 	ExternalLeading 	 the amount of extra leading that the application adds between rows.
		// 	FirstChar 			 the first character defined in the font.
		// 	Height 				 the height of characters in the font.
		// 	InternalLeading 	 the amount of leading inside the bounds set by the Height property.
		// 	Italic 				 a value indicating whether the font is italic.
		// 	LastChar 			 the last character defined in the font.
		// 	MaxCharWidth 		 the width of the widest character in the font.
		// 	Overhang 			 the extra width per string that may be added to some synthesized fonts.
		// 	PitchAndFamily 		 information about the pitch, technology, and family of a physical font.
		// 	StruckOut 			 a value indicating whether the font specifies a horizontal line through the characters.
		// 	Underlined 			 a value indicating whether the font is underlined.
		// 	Weight 				 the weight of the font.

		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
		public struct TEXTMETRICS
		{
			public int tmHeight;
			public int tmAscent;
			public int tmDescent;
			public int tmInternalLeading;
			public int tmExternalLeading;
			public int tmAveCharWidth;
			public int tmMaxCharWidth;
			public int tmWeight;
			public int tmOverhang;
			public int tmDigitizedAspectX;
			public int tmDigitizedAspectY;

			public byte tmFirstChar;
			public byte tmLastChar;
			public byte tmDefaultChar;
			public byte tmBreakChar;
			public byte tmItalic;
			public byte tmUnderlined;
			public byte tmStruckOut;
			public byte tmPitchAndFamily;
			public byte tmCharSet;
		}

		//Imported functions
		[DllImport("coredll")] static extern IntPtr SelectObject(IntPtr hdc, IntPtr hObj);
		[DllImport("coredll")] static extern bool DeleteObject(IntPtr hGdiObj);
		[DllImport("coredll", CharSet = CharSet.Unicode)] static extern int GetTextMetrics(IntPtr hdc, ref TEXTMETRICS lptm);
		
		[DllImport("gdi32", EntryPoint = "SelectObject", CharSet = CharSet.Unicode, SetLastError = true)] public static extern IntPtr SelectObject_WIN32(IntPtr hdc, IntPtr hObj);
		[DllImport("gdi32", EntryPoint = "DeleteObject", CharSet = CharSet.Unicode, SetLastError = true)] public static extern bool DeleteObject_WIN32(IntPtr hObject);
		[DllImport("gdi32", EntryPoint = "GetTextMetricsA", CharSet = CharSet.Unicode, SetLastError = true)] public static extern int GetTextMetrics_WIN32(IntPtr hdc, ref TEXTMETRICS lptm);

		#endregion

		
		
		public static void InitMetrics()
		{

			Size emPixelSize;
			//M pixel size
			int EMSize;
			//Point gerelateerd
			float dpiHorizontal;
			float dpiVertical;
			int pntLineDist;
			int pntCellHeight;

			//Niet veranderen:
			EMSize = 100;
			Font MFont = new Font("Arial", EMSize, FontStyle.Regular);
			emPixelSize = GetTextDimensions(MFont);

			//Eigenschappen vh Font ophalen
			Graphics g = Graphics.FromImage(new Bitmap(1, 1));
			dpiHorizontal = g.DpiX;
			//72.27 ?
			dpiVertical = g.DpiY;

			IntPtr hDC = g.GetHdc();
			IntPtr fhDC = MFont.ToHfont();
			IntPtr oldhDC;
			TEXTMETRICS fontMetric;

			if (TeachSettings.Instance.IsWinCE)
			{
				oldhDC = SelectObject(hDC, fhDC);
				fontMetric = new TEXTMETRICS();
				GetTextMetrics(hDC, ref fontMetric);
				//Opruimen
				SelectObject(hDC, oldhDC);
				DeleteObject(fhDC);
			}
			else
			{
				oldhDC = SelectObject_WIN32(hDC, fhDC);
				fontMetric = new TEXTMETRICS();
				GetTextMetrics_WIN32(hDC, ref fontMetric);
				//Opruimen
				SelectObject_WIN32(hDC, oldhDC);
				DeleteObject_WIN32(fhDC);
			}

			FontMetricFactors fontMetricFactors;
			
			//Bereken Factoren
			pntCellHeight = (fontMetric.tmAscent + fontMetric.tmDescent);
			pntLineDist = (pntCellHeight + fontMetric.tmExternalLeading);
			//EMSize = Cell Height minus Internal Leading
			fontMetricFactors.CharHeight = ((pntCellHeight - fontMetric.tmInternalLeading) / (float)emPixelSize.Height);
			fontMetricFactors.Cell = (pntCellHeight / (float)emPixelSize.Height);
			fontMetricFactors.Line = (pntLineDist / (float)emPixelSize.Height);
			fontMetricFactors.CharWidth = fontMetric.tmAveCharWidth / (float)emPixelSize.Height;	//M makes perfect square? (prefer emPixelSize.Width)
			fontMetricFactors.Descent = (fontMetric.tmDescent / (float)emPixelSize.Height);
			fontMetricFactors.Ascent = (fontMetric.tmAscent / (float)emPixelSize.Height);
			fontMetricFactors.InternalLeading = (fontMetric.tmInternalLeading / (float)emPixelSize.Height);
			fontMetricFactors.ExternalLeading = (fontMetric.tmExternalLeading / (float)emPixelSize.Height);

			TeachSettings.Instance.FontMetricFactors = fontMetricFactors;
			
			//'Debug info:
			//Dim myStr as new system.Text.StringBuilder()
			//myStr.Append("Em-Size" & vbTab & EMSize & vbNewLine)
			//myStr.Append("Pixel Height" & vbTab & emPixelSize.Height & vbNewLine)
			//myStr.Append("-----------------------------------------------" & vbNewLine)
			//myStr.Append("DPI verticaal" & vbTab & dpiVertical & vbNewLine)
			//myStr.Append("-----------------------------------------------" & vbNewLine)
			//myStr.Append("Height" & vbTab & FontMetric.tmHeight & vbNewLine)
			//myStr.Append("Ascent" & vbTab & FontMetric.tmAscent & vbNewLine)
			//myStr.Append("Descent" & vbTab & FontMetric.tmDescent & vbNewLine)
			//myStr.Append("Internal" & vbTab & FontMetric.tmInternalLeading & vbNewLine)
			//myStr.Append("External" & vbTab & FontMetric.tmExternalLeading & vbNewLine)
			//WriteInfo(myStr.ToString)
		}

		
		//Groottes die gebruikt worden bij text-painten
		public static void UpdatePixelFontSize()
		{
			FontPixelDimensions fontPixelDimensions = new FontPixelDimensions();
			FontMetricFactors fontMetricFactors = TeachSettings.Instance.FontMetricFactors;
			float emTextHeight = TeachSettings.Instance.EmTextHeight;
			float emTextSmallHeight = TeachSettings.Instance.EmTextHeightSmall;
			
			//Normal
			fontPixelDimensions.CellHeight = emTextHeight * fontMetricFactors.Cell;
			fontPixelDimensions.TextHeight = emTextHeight * fontMetricFactors.CharHeight;
			fontPixelDimensions.LineHeight = emTextHeight * fontMetricFactors.Line;
			fontPixelDimensions.AverageCharWidth = fontPixelDimensions.TextHeight * fontMetricFactors.CharWidth;
			fontPixelDimensions.Descent = fontPixelDimensions.TextHeight * fontMetricFactors.Descent;
			fontPixelDimensions.Ascent = fontPixelDimensions.TextHeight * fontMetricFactors.Ascent;
			fontPixelDimensions.InternalLeading = fontPixelDimensions.TextHeight * fontMetricFactors.InternalLeading;
			fontPixelDimensions.ExternalLeading = fontPixelDimensions.TextHeight * fontMetricFactors.ExternalLeading;
			
			//Small
			fontPixelDimensions.TextSmallHeight = emTextSmallHeight * fontMetricFactors.CharHeight;
			fontPixelDimensions.TextSmallOffset = (emTextHeight - emTextSmallHeight) * fontMetricFactors.Ascent;
			
			TeachSettings.Instance.FontPixelDimensions = fontPixelDimensions;
		}

		
		//TODO: onderstaande functie hoort hier niet
		public static void SetFontSize(int Height)
		{
			//load
			int emTextHeight = Height;
			int emTextSmallHeight = (int)Math.Ceiling(Height * 0.75);	//75% van normale font
			
			//TODO: onderstaande variabele aanmaken
			//Font global aanmaken zodat painten sneller gaat:
			//GlobalFontNormal = new Font("Arial", emTextHeight, FontStyle.Regular);
			//GlobalFontNormal_Bold = new Font("Arial", emTextHeight, FontStyle.Bold);
			//GlobalFontNormal_Italic = new Font("Arial", emTextHeight, FontStyle.Italic);
			//GlobalFontNormal_Underline = new Font("Arial", emTextHeight, FontStyle.Underline);
			//GlobalFontNormal_BoldItalic = new Font("Arial", emTextHeight, 9);
			//GlobalFontSmall = new Font("Arial", emTextSmallHeight, FontStyle.Regular);
			
			//save
			TeachSettings.Instance.EmTextHeight = emTextHeight;
			TeachSettings.Instance.EmTextHeightSmall = emTextSmallHeight;
		}

		
		/// <summary>
		/// Fuction that determines the size of a standard `EM-square` in pixels
		/// to determine the transformation values that we'll need to use
		/// </summary>
		/// <param name="font"></param>
		/// <returns></returns>
		public static Size GetTextDimensions(Font font)
		{

			//We doen eerst een standaard (grove) schatting.
			Bitmap b = new Bitmap(500, 500);
			Graphics gr = Graphics.FromImage(b);
			gr.Clear(Color.White);
			//We gebruiken de letter M (traditioneel EmSize) om te meten
			SizeF measuredSizeM = gr.MeasureString("M", font);
			gr.Dispose();
			b.Dispose();

			int width = (int)Math.Ceiling(measuredSizeM.Width);
			int height = (int)Math.Ceiling(measuredSizeM.Height);
			b = new System.Drawing.Bitmap(width, height, PixelFormat.Format24bppRgb);
			gr = System.Drawing.Graphics.FromImage(b);
			gr.Clear(Color.White);
			gr.DrawString("M", font, new SolidBrush(Color.Black), new Rectangle(0, 0, width, height));

			//Trucje om redelijk snel, maar precies de pixeldimensies te weten te komen:
			int x;
			int y;
			int Grens;
			Rectangle MRectangle = new Rectangle();

			//Linkerkant
			Grens = width;
			for (y = 0; y <= height - 1; y++) {
				for (x = 0; x <= Grens - 1; x++) {
					if (b.GetPixel(x, y).R != 255)
					{
						Grens = x;
					}
				}
			}
			MRectangle.X = Grens;

			//Bovenkant
			Grens = height;
			for (x = 0; x <= width - 1; x++) {
				for (y = 0; y <= Grens - 1; y++) {
					if (b.GetPixel(x, y).R != 255)
					{
						Grens = y;
					}
				}
			}
			MRectangle.Y = Grens;

			//Rechterkant
			Grens = 0;
			for (y = 0; y <= height - 1; y++) {
				for (x = width - 1; x >= Grens + 1; x += -1) {
					if (b.GetPixel(x, y).R != 255)
					{
						Grens = x;
					}
				}
			}
			MRectangle.Width = Grens - MRectangle.X;

			//Onderkant
			Grens = 0;
			for (x = 0; x <= width - 1; x++) {
				for (y = height - 1; y >= Grens + 1; y += -1) {
					if (b.GetPixel(x, y).R != 255)
					{
						Grens = y;
					}
				}
			}
			MRectangle.Height = Grens - MRectangle.Y;

			//DEBUG
			//gr.DrawRectangle(new Pen(Color.Red, 1), MRectangle)
			//b.Save("M.png",imaging.ImageFormat.Png)

			//Opruimen
			gr.Dispose();
			b.Dispose();

			return MRectangle.Size;
		}

		
	}
}
