/*
 * Copyright 2008, by Danil S. Trommel, Avance, The Netherlands
 * 
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


using System;
using System.Drawing;

using miniTeach.Teach;
using miniTeach.Tools;
using miniTeach.Settings;


//TODO: add word-wrapping functionality; cut string before
//		the last word that exceeds the screens width.

namespace miniTeach.UI
{
	/// <summary>
	/// Description of TeachPictureBox.
	/// </summary>
	public partial class TeachPictureBox : LightWeightPictureBox
	{
		
		private string PrepareMeasureString(string[] displayItems)
		{
			string measureText = StringTools.MergeStrings(displayItems);	//Glue the items in displayText together with linebreaks
			measureText = RichTools.StripRichTags(measureText);				//Remove tags
			return measureText;
		}
		
		private void InitCorrectlySizedBitmap(string measureText, int borderX, int borderY)
		{		
			Bitmap tempimage = new Bitmap(1,1);
			Graphics graphics = Graphics.FromImage(tempimage);		//Graphics object to execute the measurements on
			
			//Measure required space, using the worst case scenario (bold + italic)
			Size size = graphics.MeasureString(measureText, m_FontNormal_FullStyle).ToSize();	
			graphics.Dispose();
			
			//TODO: apply a better method of measuring! Possibly do a double draw, to determine the realized bounds
			
			//Init image
			image = new Bitmap(size.Width + 2 * borderX,
			                   size.Height + 2 * borderY);
			
			graphics = Graphics.FromImage(image);
			graphics.Clear(this.m_BackColor);
			graphics.Dispose();
		}
		

		private void DrawItem(string displayTextItem, bool isRichFormatted, ref Graphics graphics, ref float offsetX, ref float offsetY)
		{
			//Painting preferences
			Font font = new Font(TeachSettings.FontName, TeachSettings.Instance.EmTextHeight, FontStyle.Regular);
			
			if (isRichFormatted)
			{
				//Rich formatted text!
				string[] displayRichTextItems = RichTools.SplitTextAndPreserveTags(displayTextItem);
				float offsetFromBaseline = 0.0f;
				
				Brush brush = m_BrushBlack;
				
				foreach (string displayRichTextItem in displayRichTextItems)
				{
					if (displayRichTextItem.StartsWith("<")) //Tag?
					{
						//Change the paint preferences
						UpdatePaintingPreferences(displayRichTextItem, ref font, ref brush, ref offsetX, ref offsetY, ref offsetFromBaseline);
					}
					else
					{
						string text = displayRichTextItem.Trim((char)34);			//Trim quotes
						
						if (text != string.Empty)
						{
							graphics.DrawString(text, font, brush, offsetX, offsetY);	//Paint string
							offsetX += PaintTool.MeasureDisplayStringWidth(text, font);	//Update offset
						}
					}
				}
				
				//Correct any offset
				offsetY -= offsetFromBaseline;
				offsetFromBaseline = 0.0f;
			}
			else
			{
				//Just plain text
				graphics.DrawString(displayTextItem, font, m_BrushBlack, offsetX, offsetY);
			}
			
			//Update the offsets
			offsetX = this.m_TextOffset;		//N.B.: use internal field
			int numberOfLineBreaks = StringTools.CountLineBreaks(displayTextItem);
			offsetY += (numberOfLineBreaks+1)*TeachSettings.Instance.FontPixelDimensions.LineHeight; //N.B.: Always same line distance, whatever mode (sub/superscript)!
		}
		
		private void UpdatePaintingPreferences(string tag, ref Font font, ref Brush brush, ref float offsetX, ref float offsetY, ref float offsetFromBaseline)
		{
			if (tag == "<br>")
			{
				//Update offset
				offsetX = this.m_TextOffset;		//N.B.: use internal field
				offsetY += TeachSettings.Instance.FontPixelDimensions.LineHeight;		//N.B.: Always same line distance, whatever mode (sub/superscript)!
			}
			else if (tag == "<b>")
			{
				font = new Font(font.Name, font.Size, (font.Style | FontStyle.Bold));
			}
			else if (tag == "</b>")
			{
				font = new Font(font.Name, font.Size, (font.Style & ((FontStyle)0 & FontStyle.Bold)));
			}
			else if (tag == "<u>")
			{
				font = new Font(font.Name, font.Size, (font.Style | FontStyle.Underline));
			}
			else if (tag == "</u>")
			{
				font = new Font(font.Name, font.Size, (font.Style & ((FontStyle)0 & FontStyle.Underline)));
			}
			else if (tag == "<i>")
			{
				font = new Font(font.Name, font.Size, (font.Style | FontStyle.Italic));
			}
			else if (tag == "</i>")
			{
				font = new Font(font.Name, font.Size, (font.Style & ((FontStyle)0 & FontStyle.Italic)));
			}
			else if (tag == "<red>")
			{
				brush = m_BrushRed;
			}
			else if (tag == "</red>")
			{
				brush = m_BrushBlack;
			}
			else if (tag == "<def>")
			{
				brush = m_BrushBlack;
			}
			else if (tag == "<sup>")
			{
				font = new Font(font.Name, TeachSettings.Instance.EmTextHeightSmall, font.Style);
				offsetFromBaseline -= TeachSettings.Instance.FontPixelDimensions.TextSmallOffset;
				offsetY += offsetFromBaseline;
			}
			else if (tag == "</sup>")
			{
				font = new Font(font.Name, TeachSettings.Instance.EmTextHeight, font.Style);
				offsetFromBaseline += TeachSettings.Instance.FontPixelDimensions.TextSmallOffset;
				offsetY += offsetFromBaseline;
			}
			else if (tag == "<sub>")
			{
				font = new Font(font.Name, TeachSettings.Instance.EmTextHeightSmall, font.Style);
				offsetFromBaseline -= TeachSettings.Instance.FontPixelDimensions.TextSmallOffset;
				offsetY += offsetFromBaseline;
			}
			else if (tag == "</sub>")
			{
				font = new Font(font.Name, TeachSettings.Instance.EmTextHeight, font.Style);
				offsetFromBaseline += TeachSettings.Instance.FontPixelDimensions.TextSmallOffset;
				offsetY += offsetFromBaseline;
			}
			else
			{
				//Unsupported tag!
				throw new Exception("UpdatePaintingPreferences() Error: unsupported tag `" + tag + "`");
			}
		}

		
	}
}
