/*
 * Copyright 2008, by Danil S. Trommel, Avance, The Netherlands
 * 
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using miniTeach.Teach;
using miniTeach.Settings;

namespace miniTeach.UI
{
	/// <summary>
	/// Description of TeachPictureBox.
	/// </summary>
	public partial class TeachPictureBox : LightWeightPictureBox
	{
		
		private bool[] tickedCheckBoxes;		//Stores the current ticked boxes (used for painting & determining of correct answer!)
		private Rectangle[] checkBoxRectangles;
		private Rectangle[] checkBoxRectanglesInside;
		
		private void DrawQuestionMultipleChoice()
		{
			//Init offsets used to correctly offset lines etc.
			this.m_TextOffset = TeachSettings.TextOffset;
			float offsetX = this.m_TextOffset;
			float offsetY = TeachSettings.TextOffset;

			//Init bitmap, correctly sized to house the text
			string measureText = PrepareMeasureString(m_CurrentQuestion.Questions) + "\r\n" + PrepareMeasureString(m_CurrentQuestion.Answers);
			InitCorrectlySizedBitmap(measureText, (int)offsetX, (int)offsetY);	//Init bitmap to correct size

			//Paint questions
			DrawMultipleChoiceQuestions(m_CurrentQuestion.Questions, m_CurrentQuestion.QuestionUsesRichFormatting, ref offsetX, ref offsetY);

			//Add extra offset to the text for the checkboxes
			this.m_TextOffset = TeachSettings.TextOffset + (TeachSettings.Instance.FontPixelDimensions.AverageCharWidth*2);
			offsetX = this.m_TextOffset;
			
			//Below that, paint answers
			DrawMultipleChoiceQuestionsAnswers(m_CurrentQuestion.Answers, m_CurrentQuestion.AnswerUsesRichFormatting, ref offsetX, ref offsetY);
			
			//Paint boxes
			ReDrawCheckBoxes();
			
			Invalidate();		//force the control to update
		}
		
		private void DrawAnswerMultipleChoice()
		{
			//We'll only need to point the correct boxes
			Graphics graphics = Graphics.FromImage(image);	//Init graphics
			
			bool[] correctBoxes = ((MultipleChoiceQuestion)m_CurrentQuestion).MultipleChoiceAnswers;
			
			//Draw items
			for (int index = 0; index < checkBoxRectangles.Length; index++)
			{
				//Draw box in color
				if (correctBoxes[index])
				{
					graphics.FillRectangle(m_BrushGreen, checkBoxRectanglesInside[index]);
				}
				
				if (tickedCheckBoxes[index])
				{			
					if (!correctBoxes[index])
					{
						graphics.FillRectangle(m_BrushRed, checkBoxRectanglesInside[index]);
					}
					graphics.DrawRectangle(m_PenBlack, checkBoxRectangles[index]);
				}
			}
			
			graphics.Dispose();	//clean up
		}
		
		private void DrawMultipleChoiceQuestions(string[] displayItems, bool[] isRichFormatted, ref float offsetX, ref float offsetY)
		{
			Graphics graphics = Graphics.FromImage(image);	//Init graphics
			
			//Draw items
			for (int index = 0; index < displayItems.Length; index++)
			{
				DrawItem(displayItems[index], isRichFormatted[index],
				         ref graphics, ref offsetX, ref offsetY);
			}
			
			graphics.Dispose();	//clean up
		}
		
		private void DrawMultipleChoiceQuestionsAnswers(string[] displayItems, bool[] isRichFormatted, ref float offsetX, ref float offsetY)
		{
			Graphics graphics = Graphics.FromImage(image);	//Init graphics
			
			//Init boxes
			tickedCheckBoxes = new bool[displayItems.Length];
			checkBoxRectangles = new Rectangle[displayItems.Length];
			checkBoxRectanglesInside = new Rectangle[displayItems.Length];
			
			//Calculations for the position of the boxes
			int checkboxOffsetX = (int)(offsetX - (TeachSettings.Instance.FontPixelDimensions.AverageCharWidth * 1.5));
			float dy = (TeachSettings.Instance.FontPixelDimensions.InternalLeading +
			            TeachSettings.Instance.FontPixelDimensions.ExternalLeading);
			int checkboxWidth = (int)(TeachSettings.Instance.FontPixelDimensions.TextHeight -
			                          TeachSettings.Instance.FontPixelDimensions.InternalLeading);
			
			//Draw items
			for (int index = 0; index < displayItems.Length; index++)
			{
				checkBoxRectangles[index] = new Rectangle(checkboxOffsetX, (int)(offsetY+dy), checkboxWidth, checkboxWidth);
				checkBoxRectanglesInside[index] = new Rectangle(checkBoxRectangles[index].X + 2, checkBoxRectangles[index].Y + 2, checkboxWidth - 3, checkboxWidth - 3);

				DrawItem(displayItems[index], isRichFormatted[index],
				         ref graphics, ref offsetX, ref offsetY);
			}

			graphics.Dispose();	//clean up
		}
		
		private void ReDrawCheckBoxes()
		{
			Graphics graphics = Graphics.FromImage(image);	//Init graphics
			
			//Draw items
			for (int index = 0; index < checkBoxRectangles.Length; index++)
			{
				//Draw box
				graphics.DrawRectangle(m_PenGray, checkBoxRectangles[index]);
				
				//Draw filled box (inside previous) to mark selection
				if (tickedCheckBoxes[index])
				{
					graphics.FillRectangle(m_BrushBlack, checkBoxRectanglesInside[index]);
				}
				else
				{
					graphics.FillRectangle(m_BrushWipe, checkBoxRectanglesInside[index]);
				}
			}
			
			graphics.Dispose();	//clean up
		}
		
		private void CheckAnswerMultipleChoice()
		{
			m_QuestionCorrectlyAnswered = m_CurrentQuestion.IsAnswerCorrect(tickedCheckBoxes);
		}
		
		private void HandleMouseUpEvent(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			//Only interested if the current question is multiplechoice
			if (m_CurrentQuestion.QuestionType == QuestionType.MultipleChoiceSingleAnswer ||
			    m_CurrentQuestion.QuestionType == QuestionType.MultipleChoiceMultipleAnswers)
			{
				//Furthermore, the exam should be showing the question (answer doesn't require any interaction)
				if (this.m_ExamFase == ExamFase.ShowingQuestion)
				{
					//Check location
					bool didToggle = TryToToggleACheckBox(e.Y);
					
					if (didToggle)
					{
						ReDrawCheckBoxes();
						Invalidate();		//force the control to update
					}
				}
			}
		}
		
		private bool TryToToggleACheckBox(int verticalposition)
		{
			//Toggle; "...the act of switching from one to the other of two states..." ;-)
			int toggleindex = GetCheckBoxIndex(verticalposition);
			
			if (toggleindex > -1)	//valid index?
			{
				if (m_CurrentQuestion.QuestionType == QuestionType.MultipleChoiceSingleAnswer)	//Only one selection allowed
				{
					if (!tickedCheckBoxes[toggleindex])	//Index not yet ticked (= currently false)
					{
						tickedCheckBoxes = new bool[tickedCheckBoxes.Length];	//Quick re-init
						tickedCheckBoxes[toggleindex] = !tickedCheckBoxes[toggleindex];
						return true;
					}
				}
				else
				{
					tickedCheckBoxes[toggleindex] = !tickedCheckBoxes[toggleindex];
					return true;
				}
			}
			//user clicked outside area, or same toggle-situation!
			return false;
		}
		
		
		private int GetCheckBoxIndex(int verticalposition)
		{
			for(int index=0; index < checkBoxRectangles.Length; index++)
			{
				if (checkBoxRectangles[index].Top < verticalposition && verticalposition <= checkBoxRectangles[index].Bottom)
				{
					return index;
				}
			}
			
			return -1;	//user clicked outside area!
		}
		
	}
}
