/*
 * Copyright 2008, by Danil S. Trommel, Avance, The Netherlands
 * 
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using miniTeach.Teach;
using miniTeach.Tools;
using miniTeach.Settings;


namespace miniTeach.UI
{
	/// <summary>
	/// Description of TeachPictureBox.
	/// </summary>
	public partial class TeachPictureBox : LightWeightPictureBox
	{
		
		
		#region General Fields
		
		private Question m_CurrentQuestion;
		private ExamDirection m_ExamDirection;
		
		private bool m_QuestionCorrectlyAnswered;
		public bool QuestionCorrectlyAnswered {
			get { return m_QuestionCorrectlyAnswered; }
		}
		
		private ExamFase m_ExamFase;
		private float m_TextOffset;
		
		private const string RemarkSeparator = "\r\n---";
		
		#endregion
		
		
		#region Font
		
		private Font m_FontNormal;
		private Font m_FontNormal_FullStyle;
		
		#endregion
		
		
		#region Brushes & Color
		
		private Color m_BackColor;
		
		private Pen m_PenGray;
		private Pen m_PenBlack;
		
		private Brush m_BrushBlack;
		private Brush m_BrushRed;
		private Brush m_BrushGreen;
		private Brush m_BrushWipe;
		
		#endregion
		
		
		public TeachPictureBox()
		{
			//Init
			InitFont();
			InitBrushesPensColors();
			
			//Add eventhandler for click-event
			this.MouseUp += HandleMouseUpEvent;	//TODO: Efficiency: register at MC question & unregister at next non-MC question?
		}
		
		/// <summary>
		/// Updates internal font fields, used for painting
		/// Called when font size has changed (see TeachSettings)
		/// </summary>
		public void InitFont()
		{
			//Pre-initialize fonts for the sake of speed
			m_FontNormal = new Font(TeachSettings.FontName, TeachSettings.Instance.FontPixelDimensions.TextHeight, FontStyle.Regular);
			m_FontNormal_FullStyle = new Font(TeachSettings.FontName, TeachSettings.Instance.FontPixelDimensions.TextHeight, (FontStyle)9);
		}
		
		private void InitBrushesPensColors()
		{
			m_BackColor = Color.White;
			
			m_PenGray = new Pen(Color.Gray, 1);
			m_PenBlack = new Pen(Color.Black, 1);

			m_BrushBlack = new SolidBrush(Color.Black);
			m_BrushRed = new SolidBrush(Color.Red);
			m_BrushGreen = new SolidBrush(Color.Green);
			m_BrushWipe = new SolidBrush(m_BackColor);
		}
		
		
		public void NewQuestion(Question currentQuestion, ExamDirection examDirection)
		{
			//Store questioning information
			m_CurrentQuestion = currentQuestion;
			m_ExamDirection = examDirection;
			m_QuestionCorrectlyAnswered = false;
			
			ShowFase(ExamFase.ShowingQuestion);
		}
		
		public void CheckAnswer(string inputText)
		{
			//Reroute drawing to the correct function
			if (m_CurrentQuestion.QuestionType == QuestionType.Normal)
			{
				CheckAnswerNormal(inputText);
			}
			else if (m_CurrentQuestion.QuestionType == QuestionType.MultipleChoiceSingleAnswer ||
			         m_CurrentQuestion.QuestionType == QuestionType.MultipleChoiceMultipleAnswers)
			{
				CheckAnswerMultipleChoice();
			}
			else
			{
				throw new Exception("DrawQuestion() Error: unknown question type");
			}

			ShowFase(ExamFase.ShowingAnswer);
		}

		
		public void ShowFase(ExamFase examFase)
		{
			this.m_ExamFase = examFase;	//copy new fase to internal field
			
			//Ensure there is a question
			if (m_CurrentQuestion != null)
			{
				if (examFase == ExamFase.ShowingQuestion)
				{
					DrawQuestion();
				}
				else if (examFase == ExamFase.ShowingAnswer)
				{
					DrawAnswer();
				}
				else if (examFase == ExamFase.AskingRestart)
				{
					DrawRestartInfo();
				}
			}
			
			base.Refresh();
		}
		
		
		private void DrawQuestion()
		{
			//Reroute drawing to the correct function
			if (m_CurrentQuestion.QuestionType == QuestionType.Normal)
			{
				DrawQuestionNormal();
			}
			else if (m_CurrentQuestion.QuestionType == QuestionType.MultipleChoiceSingleAnswer ||
			         m_CurrentQuestion.QuestionType == QuestionType.MultipleChoiceMultipleAnswers)
			{
				DrawQuestionMultipleChoice();
			}
			else
			{
				throw new Exception("DrawQuestion() Error: unknown question type");
			}
		}
		
		private void DrawAnswer()
		{
			//Reroute drawing to the correct function
			if (m_CurrentQuestion.QuestionType == QuestionType.Normal)
			{
				DrawAnswerNormal();
			}
			else if (m_CurrentQuestion.QuestionType == QuestionType.MultipleChoiceSingleAnswer ||
			         m_CurrentQuestion.QuestionType == QuestionType.MultipleChoiceMultipleAnswers)
			{
				DrawAnswerMultipleChoice();
			}
			else
			{
				throw new Exception("DrawQuestion() Error: unknown question type");
			}
		}
		
		public void DrawRestartInfo()
		{
			string[] restartinfo = new string[]{"The exam finished.\r\n\r\nPress |<< to restart..."};
			bool[] richformatted = new bool[]{false};
			DrawNormal(restartinfo, richformatted);
		}
		
		
	}
}
